import os
from pathlib import Path
from file_thumbnailer.ConverterManager import ConverterManager
from file_thumbnailer.models.Dimensions import Dimensions


def test_converter_manager_init() -> None:
    ConverterManager()


def test_get_convert_pdf() -> None:
    converter_manager = ConverterManager()
    files_path = Path(os.path.join(os.path.dirname(__file__), 'files'))

    with open(os.path.join(files_path, 'sample.pdf'), 'rb') as sample_pdf:
        processor = converter_manager.from_data(sample_pdf.read())
    image_bytes = processor.to_image_bytes(Dimensions(height=500))

    assert isinstance(image_bytes, bytes)


def test_get_convert_epub() -> None:
    converter_manager = ConverterManager()
    files_path = Path(os.path.join(os.path.dirname(__file__), 'files'))

    processor = converter_manager.from_file(os.path.join(files_path, 'sample.epub'))
    image_bytes = processor.to_image_bytes(Dimensions(height=500))

    assert isinstance(image_bytes, bytes)


def test_get_convert_bmp() -> None:
    converter_manager = ConverterManager()
    files_path = Path(os.path.join(os.path.dirname(__file__), 'files'))

    processor = converter_manager.from_file(os.path.join(files_path, 'sample.bmp'))
    image_bytes = processor.to_image_bytes(Dimensions(height=500))

    assert isinstance(image_bytes, bytes)



from typing import Optional
import mimetypes
import pathlib
import magic
from file_thumbnailer.models.Dimensions import Dimensions


class Tools:

    @staticmethod
    def calculate_dimensions(from_dimensions: Dimensions, to_dimensions: Dimensions) -> Dimensions:
        ratio_from = from_dimensions.width / from_dimensions.height

        if not to_dimensions.width and to_dimensions.height:
            to_dimensions.width = int(to_dimensions.height * ratio_from)

        if not to_dimensions.height and to_dimensions.width:
            to_dimensions.height = int(to_dimensions.width / ratio_from)

        ratio_to = to_dimensions.width / to_dimensions.height

        if ratio_from > ratio_to:
            ratio = to_dimensions.width / from_dimensions.width
        else:
            ratio = to_dimensions.height / from_dimensions.height

        return Dimensions(
            round(from_dimensions.width * ratio),
            round(from_dimensions.height * ratio)
        )

    @staticmethod
    def detect_mimetype(data: bytes, file_path: Optional[pathlib.Path] = None) -> str:
        extension_detection_mimetypes = [
            'application/zip',
            'text/xml'
        ]

        mime_type = magic.from_buffer(data, mime=True)  # type: ignore
        if file_path and mime_type in extension_detection_mimetypes:
            mime_type, _ = mimetypes.guess_type(str(file_path.absolute()))

        return mime_type



import io
from typing import List, Optional
from PIL import Image
from file_thumbnailer.models.Dimensions import Dimensions
from file_thumbnailer.Tools import Tools


class Converter:

    def __init__(self, data: bytes, mime_type: str):
        self.data = data
        self.mime_type = mime_type

    @staticmethod
    def is_available() -> bool:
        raise NotImplementedError

    @staticmethod
    def get_handle_mimetypes() -> List[str]:
        raise NotImplementedError

    def to_pil_image(self, page: Optional[int] = None) -> Image:
        raise NotImplementedError

    def to_image_bytes(self, dimensions: Dimensions, image_format: str = 'PNG') -> bytes:
        pil_image = self.to_pil_image()
        buffer = io.BytesIO()
        calculated_dimensions = Tools.calculate_dimensions(Dimensions(pil_image.width, pil_image.height), dimensions)
        resized = pil_image.resize((calculated_dimensions.width, calculated_dimensions.height), resample=True)
        resized.save(buffer, format=image_format)
        return buffer.getvalue()



import io
from typing import List, Optional
from file_thumbnailer.converters.Converter import Converter

is_pil = True
try:
    from PIL import Image
except ImportError:
    is_pil = False

is_fitz = True
try:
    import fitz
except ImportError:
    is_fitz = False


class PdfConverter(Converter):

    @staticmethod
    def is_available() -> bool:
        return is_fitz and is_pil

    @staticmethod
    def get_handle_mimetypes() -> List[str]:
        return [
            'application/pdf',
            'application/vnd.ms-xpsdocument',
            'application/oxps',
            'application/epub+zip',
            'application/vnd.comicbook+zip'
        ]

    def to_pil_image(self, page: Optional[int] = None) -> Image:
        if not page:
            page = 0
        source_pdf = fitz.Document(stream=self.data, filetype=self.mime_type)
        selected_page: fitz.Page = source_pdf[page]

        # Compatibility with older version of fitz
        pixmap_callable = getattr(selected_page, 'get_pixmap', getattr(selected_page, 'getPixmap'))

        selected_page_image = pixmap_callable(alpha=False, matrix=fitz.Matrix(4.0, 4.0))
        # Compatibility with older version of fitz
        tobytes_callable = getattr(selected_page_image, 'tobytes', getattr(selected_page_image, 'getPNGData'))
        return Image.open(io.BytesIO(tobytes_callable()))



import io
from typing import List, Optional
from file_thumbnailer.converters.Converter import Converter

is_pil = True
try:
    from PIL import Image
except ImportError:
    is_pil = False


class ImgConverter(Converter):

    @staticmethod
    def is_available() -> bool:
        return is_pil

    @staticmethod
    def get_handle_mimetypes() -> List[str]:
        return [
            'image/bmp',
            'image/vnd-ms.dds',
            'image/dib',
            'application/eps',
            'image/gif',
            'image/jpeg',
            'image/png',
            'image/ico',
            'image/webp',
        ]

    def to_pil_image(self, page: Optional[int] = None) -> Image:
        return Image.open(io.BytesIO(self.data))



import logging
from typing import Dict, Type, Optional
from pathlib import Path
from file_thumbnailer.converters.Converter import Converter
from file_thumbnailer.converters.PdfConverter import PdfConverter
from file_thumbnailer.converters.ImgConverter import ImgConverter
from file_thumbnailer.Tools import Tools
from file_thumbnailer.exceptions import NotSupportedException


class ConverterManager:
    log = logging.getLogger(__name__)
    supported_mimetypes: Dict[str, Type[Converter]] = {}
    converter_classes = [
        PdfConverter,
        ImgConverter
    ]

    def __init__(self) -> None:
        # Make sure that self.supported_mimetypes is empty, things got weird when running in pytest (supported_mimetypes was already full when running bellow code)
        self.supported_mimetypes = {}
        for converter_class in self.converter_classes:
            if not converter_class.is_available():
                self.log.info('Converter %s is not available', converter_class)
                continue
            for supported_mimetype in converter_class.get_handle_mimetypes():
                mimetype_used = self.supported_mimetypes.get(supported_mimetype)
                if mimetype_used:
                    raise Exception('Mimetype is already handled by {}'.format(mimetype_used))

                self.supported_mimetypes[supported_mimetype] = converter_class

        self.log.info('Supported mimetypes: %s', self.supported_mimetypes.keys())

    def from_data(self, data: bytes, force_mime_type: Optional[str] = None) -> Converter:
        mime_type = Tools.detect_mimetype(data) if not force_mime_type else force_mime_type
        converter = self.supported_mimetypes.get(mime_type)
        if not converter:
            raise NotSupportedException('Mimetype {} is not supported, supported mimes are: {}'.format(mime_type, ', '.join(self.supported_mimetypes.keys())))
        return converter(data, mime_type)

    def from_file(self, file_path: str, force_mime_type: Optional[str] = None) -> Converter:
        path_info = Path(file_path)
        with path_info.open('rb') as file_handle:
            data = file_handle.read()
            mime_type = Tools.detect_mimetype(data, path_info) if not force_mime_type else force_mime_type
            return self.from_data(data, mime_type)
